#include "net.h"

#include <unistd.h> /* close(), read() */
#include <sys/types.h> /* socket(), bind(), accept(), read() */
#include <sys/socket.h> /* socket(), bind(), listen(), accept() */
#include <string.h> /* memset(), strlen() */
#include <netinet/in.h> /* htons(), htonl() */
#include <fcntl.h> /* fcntl() */
#include <stdlib.h> /* malloc() */
#include <netdb.h> /* gethostbyname() */
#include <stdio.h> /* fprintf(), stderr */
#include <arpa/inet.h> /* inet_ntoa() */

#define DEBUG(x) {fprintf(stderr, "%s", x); fflush(stderr);}

/*************************************************** port ********/

port::port(int number)
{    struct sockaddr_in s;

     memset( (char *) &s, 0, sizeof(s) );
     s.sin_port = htons(number);
     s.sin_addr.s_addr = htonl(INADDR_ANY);
     s.sin_family = AF_INET;

     if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) return;

     if (bind(sock, (struct sockaddr *) &s, sizeof(s)) < 0)
     {   close(sock); 
         sock = -1; /* mark error */
         return;
         }
     listen(sock, 6);
     }

port::~port(void)
{    close(sock);
     }

int port::operator !(void)
{    return (sock == -1);
     }

/*************************************************** host ********/

host::host(char *addr, int p)
{   struct hostent *h;
    struct in_addr a;
    struct sockaddr_in sin;
 
    if (!((*addr >= '0') && (*addr <= '9')))
    {   h = gethostbyname(addr);
        if (h == NULL)
        {   fprintf(stderr, "Couldn't find host %s .\n", addr);
            sock = -1;
            return;
            }
        addr = (char *) inet_ntoa(*(struct in_addr *) h->h_addr_list[0]);
        }

    a.s_addr = inet_addr(addr);
    if ( (int) a.s_addr == -1)
    {   fprintf(stderr, "Couldn't find host %s .\n", addr);
        sock = -1;
        return;
        }

    memset( (char *) &sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_port = htons(p);
    memcpy( (char *) &sin.sin_addr, &a, sizeof(a));

    if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
    {   sock = -1;
        return; 
        }

    if(connect(sock, (struct sockaddr *) &sin, sizeof(sin)) < 0) 
    {   sock = -1;
        return; 
        }

    fcntl(sock, F_SETFL, O_NONBLOCK);
    }

host::~host(void)
{    close(sock);
     }

int host::operator !(void)
{    return (sock == -1);
     }

char host::get_one_char(void)
{   char c[1];
    if (read(sock, c, 1) == -1)
       return 0;
    return *c; 
    }

char *host::getline(void)
{   char *buffer, *count;

    buffer = (char *) malloc(1000);
    count = buffer;

    while (*count = get_one_char())
    {   if ((*count == '\n') || (count - buffer > 998))
        {   *(count+1) = 0;
            return buffer;
            }
        if (*count != '\r') count++;
        }
    return buffer;
    }
         
void host::sendline(char *c)
{   write(sock, c, strlen(c));
    }

void host::sendchar(char c)
{   write(sock, &c, 1);
    if (c == 13) sendchar(10);
    }
